@@ -22,10 +22,8 @@
 #' }
 #' 
 #' @seealso 
-#' \code{\link[brms:cor_arma]{cor_arma}, \link[brms:cor_ar]{cor_ar},
-#'       \link[brms:cor_ma]{cor_ma}, \link[brms:cor_arr]{cor_arr}, 
-#'       \link[brms:cor_car]{cor_car}, \link[brms:cor_sar]{cor_sar},
-#'       \link[brms:cor_bsts]{cor_bsts}, \link[brms:cor_fixed]{cor_fixed}}
+#' \code{\link{cor_arma}, \link{cor_ar}, \link{cor_ma}, \link{cor_arr}, 
+#'       \link{cor_car}, \link{cor_sar}, \link{cor_bsts}, \link{cor_fixed}}
 #' 
 NULL
 
@@ -68,8 +66,7 @@ NULL
 #' 
 #' @author Paul-Christian Buerkner \email{paul.buerkner@@gmail.com}
 #' 
-#' @seealso \code{\link[brms:cor_ar]{cor_ar}} \code{\link[brms:cor_ma]{cor_ma}}
-#'   \code{\link[brms:cor_arr]{cor_arr}}
+#' @seealso \code{\link{cor_ar}, \link{cor_ma}, \link{cor_arr}}
 #' 
 #' @examples
 #' cor_arma(~visit|patient, p = 2, q = 2)
@@ -105,6 +102,8 @@ cor_arma <- function(formula = ~ 1, p = 0, q = 0, r = 0, cov = FALSE) {
 
 #' 
 #' @inheritParams cor_arma
+#' @param p A non-negative integer specifying the autoregressive (AR) 
+#'   order of the ARMA structure. Default is 1.  
 #' 
 #' @return An object of class \code{cor_arma} containing solely autoregression terms.
 #' 
@@ -131,6 +130,8 @@ cor_ar <- function(formula = ~ 1, p = 1, cov = FALSE) {
 #' allowing for moving average terms only.
 #' 
 #' @inheritParams cor_arma
+#' @param q A non-negative integer specifying the moving average (MA) 
+#'   order of the ARMA structure. Default is 1.  
 #' 
 #' @return An object of class \code{cor_arma} containing solely moving average terms.
 #' 
@@ -152,6 +153,8 @@ cor_ma <- function(formula = ~ 1, q = 1, cov = FALSE) {
 #' allowing for autoregressive effects of the response only.
 #' 
 #' @inheritParams cor_arma
+#' @param r A non-negative integer specifying the autoregressive
+#'   response (ARR) order of the ARMA structure. Default is 1.  
 #' 
 #' @return An object of class \code{cor_arma} containing solely
 #'   autoregressive response terms.
@@ -196,7 +199,7 @@ cor_arr <- function(formula = ~ 1, r = 1) {
 #'   support SAR structures.
 #' 
 #' @return An object of class \code{cor_sar} to be used in calls to
-#'   \code{\link[brms:brm]{brm}}.
+#'   \code{\link{brm}}.
 #'   
 #' @examples 
 #' \dontrun{
@@ -577,6 +580,38 @@ remove_autocor <- function(x) {
   x
 }
 
+subset_autocor <- function(x, subset, autocor = NULL) {
+  # subset matrices stored in cor_brms objects
+  # Args:
+  #   x: a brmsfit object to be updated
+  #   subset: indices of observations to keep
+  #   autocor: optional (list of) cor_brms objects
+  #     from which to take matrices
+  # Returns:
+  #   an updated brmsfit object
+  .subset_autocor <- function(autocor) {
+    if (is.cor_sar(autocor)) {
+      autocor$W <- autocor$W[subset, subset, drop = FALSE]
+    } else if (is.cor_fixed(autocor)) {
+      autocor$V <- autocor$V[subset, subset, drop = FALSE]
+    }
+    return(autocor)
+  }
+  if (is.null(autocor)) {
+    autocor <- autocor(x)
+  }
+  if (is_mv(x)) {
+    for (i in seq_along(x$formula$forms)) {
+      new_autocor <- .subset_autocor(autocor[[i]])
+      x$formula$forms[[i]]$autocor <- x$autocor[[i]] <- new_autocor
+    }
+  } else {
+    x$formula$autocor <- x$autocor <- .subset_autocor(autocor)
+  }
+  # prevents double updating in add_new_objects()
+  structure(x, autocor_updated = TRUE)
+}
+
 regex_cor_pars <- function() {
   # regex to extract all pars of cor structures
   # used in summary.brmsfit
