@@ -3,6 +3,11 @@ extract_draws <- function(x, ...) {
   UseMethod("extract_draws")
 }
 
+#' @export
+extract_draws.default <- function(x, ...) {
+  NULL
+}
+
 #' @export
 extract_draws.brmsfit <- function(x, newdata = NULL, re_formula = NULL, 
                                   sample_new_levels = "uncertainty",
@@ -21,24 +26,34 @@ extract_draws.brmsfit <- function(x, newdata = NULL, re_formula = NULL,
   if (!incl_autocor) {
     x <- remove_autocor(x) 
   }
-  sdata <- validate_newdata(
-    newdata, x, re_formula = re_formula, resp = resp, ...
-  )
   subset <- subset_samples(x, subset, nsamples)
   samples <- as.matrix(x, subset = subset)
+  # prepare (new) data and stan data 
+  new <- !is.null(newdata)
+  newd_args <- nlist(newdata, object = x, re_formula, resp, ...)
+  newdata <- do.call(validate_newdata, newd_args)
+  newd_args$newdata <- newdata
+  newd_args$internal <- TRUE
+  sdata <- do.call(standata, newd_args)
   new_formula <- update_re_terms(x$formula, re_formula)
   bterms <- parse_bf(new_formula)
-  ranef <- tidy_ranef(bterms, data = x$data)
+  ranef <- tidy_ranef(bterms, x$data)
+  meef <- tidy_meef(bterms, x$data)
   args <- nlist(
     x = bterms, samples, sdata, data = x$data,
-    ranef, old_ranef = x$ranef, sample_new_levels,
-    resp, nug, smooths_only, new = !is.null(newdata)
+    ranef, old_ranef = x$ranef, meef, resp,
+    sample_new_levels, nug, smooths_only, new,
+    stanvars = names(x$stanvars)
   )
-  if (length(get_effect(bterms, "gp")) && !is.null(newdata)) {
-    # GPs for new data require the original data as well
-    args$old_sdata <- validate_newdata(
-      newdata = NULL, fit = x, re_formula = re_formula, ...
-    )
+  if (new) {
+    # extract_draws_re() also requires the new level names
+    # original level names are already passed via old_ranef
+    new_levels <- attr(tidy_ranef(bterms, newdata), "levels")
+    attr(args$ranef, "levels") <- new_levels
+    if (length(get_effect(bterms, "gp"))) {
+      # GPs for new data require the original data as well
+      args$old_sdata <- standata(x, internal = TRUE, ...)
+    }
   }
   do.call(extract_draws, args)
 }
@@ -128,10 +143,15 @@ extract_draws.brmsterms <- function(x, samples, sdata, ...) {
 
 #' @export
 extract_draws.btnl <- function(x, samples, sdata, ...) {
-  # Args:
-  #   C: matrix containing covariates
-  #   ...: passed to extract_draws.btl
-  draws <- list(f = x$family, nsamples = nrow(samples), nobs = sdata$N)
+  draws <- list(
+    f = x$family, nlform = x$formula[[2]],
+    nsamples = nrow(samples), nobs = sdata$N
+  )
+  class(draws) <- "bdrawsnl"
+  if (is_nlpar(x)) {
+    draws$nlpar <- x$nlpar
+    return(draws)
+  }
   nlpars <- names(x$nlpars)
   for (nlp in nlpars) {
     draws$nlpars[[nlp]] <- extract_draws(
@@ -146,136 +166,192 @@ extract_draws.btnl <- function(x, samples, sdata, ...) {
       C[, cov], dim = c(nrow(samples), nrow(C))
     )
   }
-  draws$nlform <- x$formula[[2]]
-  draws$f <- x$family
   draws
 }
 
 #' @export
 extract_draws.btl <- function(x, samples, sdata, smooths_only = FALSE, ...) {
-  # extract draws of all kinds of effects
-  # Args:
-  #   fit: a brmsfit object
-  #   mv: is the model multivariate?
-  #   ...: further elements to store in draws
-  #   other arguments: see extract_draws.brmsfit
-  # Returns:
-  #   A named list to be interpreted by linear_predictor
   nsamples <- nrow(samples)
   draws <- nlist(f = x$family, nsamples, nobs = sdata$N)
+  class(draws) <- "bdrawsl"
   args <- nlist(bterms = x, samples, sdata, ...)
   draws$fe <- do.call(extract_draws_fe, args)
   draws$sm <- do.call(extract_draws_sm, args)
   if (smooths_only) {
     # make sure only smooth terms will be included in draws
-    return(structure(draws, predicted = TRUE))
+    return(draws)
   }
-  draws$mo <- do.call(extract_draws_mo, args)
+  draws$sp <- do.call(extract_draws_sp, args)
   draws$cs <- do.call(extract_draws_cs, args)
-  draws$me <- do.call(extract_draws_me, args)
   draws$gp <- do.call(extract_draws_gp, args)
   draws$re <- do.call(extract_draws_re, args)
   draws$offset <- do.call(extract_draws_offset, args)
   if (!(use_cov(x$autocor) || is.cor_sar(x$autocor))) {
     draws$ac <- do.call(extract_draws_autocor, args)
   }
-  structure(draws, predicted = TRUE)
+  draws
 }
 
 extract_draws_fe <- function(bterms, samples, sdata, ...) {
   # extract draws of ordinary population-level effects
-  # Args:
-  #   fixef: names of the population-level effects
-  #   args: list of arguments passed to as.matrix.brmsfit
-  #   nlpar: name of a non-linear parameter
-  #   old_cat: see old_categorical
-  # Returns: 
-  #   A named list to be interpreted by linear_predictor
-  # stopifnot("x" %in% names(args))
   draws <- list()
   p <- usc(combine_prefix(bterms))
-  draws$X <- sdata[[paste0("X", p)]]
-  fixef <- colnames(draws$X)
+  X <- sdata[[paste0("X", p)]]
+  fixef <- colnames(X)
   if (length(fixef)) {
+    draws$X <- X
     b_pars <- paste0("b", p, "_", fixef)
     draws$b <- get_samples(samples, b_pars, exact = TRUE)
   }
   draws
 }
 
-extract_draws_mo <- function(bterms, samples, sdata, data, ...) {
-  # extract draws of monotonic effects
-  # Args:
-  #   monef: names of the monotonic effects
-  #   args: list of arguments passed to as.matrix.brmsfit
-  #   sdata: list returned by make_standata
-  # Returns: 
-  #   A named list to be interpreted by linear_predictor
+extract_draws_sp <- function(bterms, samples, sdata, data, 
+                             meef, new = FALSE, ...) {
+  # extract draws of special effects terms
   draws <- list()
-  monef <- get_mo_labels(bterms, data)
-  if (!length(monef)) {
+  spef <- tidy_spef(bterms, data)
+  if (is.null(spef)) {
     return(draws) 
   }
   p <- usc(combine_prefix(bterms))
-  att <- attributes(monef)
-  # prepare calls evaluated in 'mo_predictor()'
-  calls <- rep(NA, length(monef))
-  for (i in seq_along(calls)) {
-    tmp <- paste0(".mo(simo_", att$Imo[[i]], ", Xmo_", att$Imo[[i]], ")")
-    calls[i] <- paste0(tmp, collapse = " * ")
-    if (att$not_one[i]) {
-      str_add(calls[i]) <- paste0(" * Cmo_", att$Icmo[i])
+  # prepare calls evaluated in sp_predictor
+  draws$calls <- vector("list", nrow(spef))
+  for (i in seq_along(draws$calls)) {
+    call <- spef$call_prod[[i]]
+    if (!is.null(spef$call_mo[[i]])) {
+      new_mo <- paste0(
+        ".mo(simo_", spef$Imo[[i]], ", Xmo_", spef$Imo[[i]], ")"
+      )
+      call <- rename(call, spef$call_mo[[i]], new_mo)
     }
+    if (!is.null(spef$call_me[[i]])) {
+      new_me <- paste0("Xme_", seq_along(meef$term))
+      call <- rename(call, meef$term, new_me)
+    }
+    if (!is.null(spef$call_mi[[i]])) {
+      new_mi <- paste0("Yl_", spef$vars_mi[[i]])
+      call <- rename(call, spef$call_mi[[i]], new_mi)
+    }
+    if (spef$Ic[i] > 0) {
+      str_add(call) <- paste0(" * Csp_", spef$Ic[i])
+    }
+    draws$calls[[i]] <- parse(text = paste0(call))
   }
-  draws$calls <- parse(text = paste0(calls))
-  # extract data and parameters for monotonic effects
-  monef <- rename(monef)
-  draws$bmo <- named_list(monef)
-  for (i in seq_along(monef)) {
-    bmo_par <- paste0("bmo", p, "_", monef[i])
-    draws$bmo[[i]] <- get_samples(samples, bmo_par, exact = TRUE)
-  }
-  simo_coef <- get_simo_labels(monef)
+  # extract general data and parameters for special effects
+  bsp_pars <- paste0("bsp", p, "_", spef$coef)
+  draws$bsp <- get_samples(samples, bsp_pars, exact = TRUE)
+  # prepare draws specific to monotonic effects
+  simo_coef <- get_simo_labels(spef)
   draws$simo <- draws$Xmo <- named_list(simo_coef)
   for (i in seq_along(simo_coef)) {
-    simo_par <- paste0(
-      "simo", p, "_", simo_coef[i], "[", seq_len(sdata$Jmo[i]), "]"
-    )
+    J <- seq_len(sdata$Jmo[i])
+    simo_par <- paste0("simo", p, "_", simo_coef[i], "[", J, "]")
     draws$simo[[i]] <- get_samples(samples, simo_par, exact = TRUE)
     draws$Xmo[[i]] <- sdata[[paste0("Xmo", p, "_", i)]]
   }
-  ncovars <- sum(att$not_one)
-  dim <- c(nrow(draws$bmo[[1]]), sdata$N)
-  for (i in seq_len(ncovars)) {
-    draws$Cmo[[i]] <- as_draws_matrix(
-      sdata[[paste0("Cmo", p, "_", i)]], dim = dim
+  # prepare draws specific to noise-free effects
+  warn_me <- FALSE
+  if (nrow(meef)) {
+    save_mevars <- any(grepl("^Xme_", colnames(samples)))
+    warn_me <- warn_me || !new && !save_mevars
+    draws$Xme <- named_list(meef$coef)
+    Xme_pars <- paste0("Xme_", escape_all(meef$coef), "\\[")
+    Xn <- sdata[paste0("Xn_", seq_len(nrow(meef)))]
+    noise <- sdata[paste0("noise_", seq_len(nrow(meef)))]
+    groups <- unique(meef$grname)
+    for (i in seq_along(groups)) {
+      g <- groups[i]
+      K <- which(meef$grname %in% g)
+      if (nzchar(g)) {
+        Jme <- sdata[[paste0("Jme_", i)]]
+        me_dim <- c(nrow(draws$bsp), length(unique(Jme)))
+      } else {
+        me_dim <- c(nrow(draws$bsp), sdata$N)
+      }
+      if (!new && save_mevars) {
+        # extract original samples of latent variables
+        for (k in K) {
+          draws$Xme[[k]] <- get_samples(samples, Xme_pars[k])
+        }
+      } else {
+        # sample new values of latent variables
+        for (k in K) {
+          dXn <- as_draws_matrix(Xn[[k]], me_dim)
+          dnoise <- as_draws_matrix(noise[[k]], me_dim)
+          draws$Xme[[k]] <- array(rnorm(prod(me_dim), dXn, dnoise), me_dim)
+          remove(dXn, dnoise)
+        }
+      }
+      if (nzchar(g)) {
+        for (k in K) {
+          draws$Xme[[k]] <- draws$Xme[[k]][, Jme]
+        }
+      }
+    }
+  }
+  # prepare draws specific to missing value variables
+  dim <- c(nrow(draws$bsp), sdata$N)
+  vars_mi <- unique(unlist(spef$vars_mi))
+  if (length(vars_mi)) {
+    resps <- usc(vars_mi)
+    Yl_names <- paste0("Yl", resps)
+    draws$Yl <- named_list(Yl_names)
+    for (i in seq_along(draws$Yl)) {
+      Y <- as_draws_matrix(sdata[[paste0("Y", resps[i])]], dim)
+      sdy <- sdata[[paste0("noise", resps[i])]]
+      if (is.null(sdy)) {
+        # missings only
+        draws$Yl[[i]] <- Y
+        if (!new) {
+          Ymi_pars <- paste0("Ymi", resps[i], "\\[")
+          Ymi <- get_samples(samples, Ymi_pars)
+          Jmi <- sdata[[paste0("Jmi", resps[i])]]
+          draws$Yl[[i]][, Jmi] <- Ymi
+        }
+      } else {
+        # measurement-error in the response
+        save_mevars <- any(grepl("^Yl_", colnames(samples)))
+        if (save_mevars && !new) {
+          Yl_pars <- paste0("Yl", resps[i], "\\[")
+          draws$Yl[[i]] <- get_samples(samples, Yl_pars)
+        } else {
+          warn_me <- warn_me || !new
+          sdy <- as_draws_matrix(sdy, dim)
+          draws$Yl[[i]] <- array(rnorm(prod(dim), Y, sdy), dim)
+        }
+      }
+    }
+  }
+  if (warn_me) {
+    warning2(
+      "Noise-free variables were not saved. Please set ",
+      "argument 'save_mevars' to TRUE when fitting your model. ",
+      "Treating original data as if it was new data as a workaround."
     )
   }
+  # prepare covariates
+  ncovars <- max(spef$Ic)
+  for (i in seq_len(ncovars)) {
+    draws$Csp[[i]] <- sdata[[paste0("Csp", p, "_", i)]]
+    draws$Csp[[i]] <- as_draws_matrix(draws$Csp[[i]], dim = dim)
+  }
   draws
 }
 
 extract_draws_cs <- function(bterms, samples, sdata, data, ...) {
-  # category specific effects 
   # extract draws of category specific effects
-  # Args:
-  #   csef: names of the category specific effects
-  #   args: list of arguments passed to as.matrix.brmsfit
-  #   nlpar: name of a non-linear parameter
-  #   old_cat: see old_categorical
-  # Returns: 
-  #   A named list to be interpreted by linear_predictor
   draws <- list()
   p <- usc(combine_prefix(bterms))
+  resp <- usc(bterms$resp)
   int_regex <- paste0("^b", p, "_Intercept\\[")
   is_ordinal <- any(grepl(int_regex, colnames(samples))) 
   if (is_ordinal) {
-    draws$ncat <- sdata[[paste0("ncat", p)]]
+    draws$ncat <- sdata[[paste0("ncat", resp)]]
     draws$Intercept <- get_samples(samples, int_regex)
     csef <- colnames(get_model_matrix(bterms$cs, data))
     if (length(csef)) {
-      # as of brms > 1.0.1 the original prefix 'bcs' is used
-      bcs <- ifelse(any(grepl("^bcs_", colnames(samples))), "^bcs", "^b")
-      cs_pars <- paste0(bcs, p, "_", csef, "\\[")
+      cs_pars <- paste0("^bcs", p, "_", csef, "\\[")
       draws$bcs <- get_samples(samples, cs_pars)
       draws$Xcs <- sdata[[paste0("Xcs", p)]]
     }
@@ -283,74 +359,8 @@ extract_draws_cs <- function(bterms, samples, sdata, data, ...) {
   draws
 }
 
-extract_draws_me <- function(bterms, samples, sdata, data, 
-                             new = FALSE, ...) {
-  # extract draws of noise-free effects
-  # Args:
-  #   meef: names of the noise free effects
-  #   args: list of arguments passed to as.matrix.brmsfit
-  #   sdata: list returned by make_standata
-  #   nlpar: name of a non-linear parameter
-  #   new: logical; are new data specified?
-  # Returns: 
-  #   A named list to be interpreted by linear_predictor
-  draws <- list()
-  meef <- get_me_labels(bterms, data)
-  if (!length(meef)) {
-    return(draws) 
-  }
-  p <- usc(combine_prefix(bterms))
-  if (new) {
-    stop2("Predictions with noise-free variables are not yet ",
-          "possible when passing new data.")
-  }
-  if (!any(grepl("^Xme_", colnames(samples)))) {
-    stop2("Noise-free variables were not saved. Please set ",
-          "argument 'save_mevars' to TRUE when fitting your model.")
-  }
-  att <- attributes(meef)
-  uni_me <- att$uni_me
-  # prepare calls to evaluate noise-free data
-  meef_terms <- ulapply(att$calls_me, paste0, collapse = " * ")
-  new_me <- paste0("Xme_", seq_along(uni_me))
-  meef_terms <- rename(meef_terms, uni_me, new_me)
-  covars <- ifelse(att$not_one, paste0(" * Cme_", att$Icme), "")
-  meef_terms <- paste0(meef_terms, covars)
-  draws$calls <- parse(text = meef_terms)
-  # extract coefficient samples
-  meef <- rename(meef)
-  draws$bme <- named_list(meef)
-  for (j in seq_along(draws[["bme"]])) {
-    bme_par <- paste0("bme", p, "_", meef[j])
-    draws$bme[[j]] <- get_samples(samples, bme_par, exact = TRUE)
-  }
-  # extract noise-free variable samples
-  uni_me <- rename(uni_me)
-  draws$Xme <- named_list(uni_me)
-  for (j in seq_along(draws[["Xme"]])) {
-    Xme_pars <- paste0("Xme", p, "_", uni_me[j], "\\[")
-    draws$Xme[[j]] <- get_samples(samples, Xme_pars)
-  }
-  # prepare covariates
-  ncovars <- sum(att$not_one)
-  for (j in seq_len(ncovars)) {
-    cme <- paste0("Cme", p, "_", j)
-    draws$Cme[[j]] <- as_draws_matrix(
-      sdata[[cme]], dim = dim(draws[["Xme"]][[1]])
-    )
-  }
-  draws
-}
-
 extract_draws_sm <- function(bterms, samples, sdata, data, ...) {
   # extract draws of smooth terms
-  # Args:
-  #   smooths: names of the smooth terms
-  #   args: list of arguments passed to as.matrix.brmsfit
-  #   sdata: list returned by make_standata
-  #   nlpar: name of a non-linear parameter
-  # Returns: 
-  #   A named list to be interpreted by linear_predictor
   smooths <- get_sm_labels(bterms, data, covars = TRUE)
   if (!length(smooths)) {
     return(list())
@@ -373,9 +383,11 @@ extract_draws_sm <- function(bterms, samples, sdata, data, ...) {
 extract_draws_gp <- function(bterms, samples, sdata, data,
                              new = FALSE, nug = NULL,
                              old_sdata = NULL, ...) {
-  # extract draws for gaussian processes
+  # extract draws for Gaussian processes
   # Args:
-  #   gpef: names of the gaussian process terms
+  #   new: Is new data used?
+  #   nug: small numeric value to avoid numerical problems in GPs
+  #   old_sdata: standata object based on the original data
   gpef <- get_gp_labels(bterms, data, covars = TRUE)
   if (!length(gpef)) {
     return(list()) 
@@ -425,12 +437,8 @@ extract_draws_re <- function(bterms, samples, sdata, data, ranef, old_ranef,
                              sample_new_levels = "uncertainty", ...) {
   # extract draws of group-level effects
   # Args:,
-  #   ranef: data.frame returned by tidy_ranef
-  #   args: list of arguments passed to as.matrix.brmsfit
-  #   sdata: list returned by make_standata
-  #   sample_new_levels: see help("predict.brmsfit")
-  # Returns: 
-  #   A named list to be interpreted by linear_predictor
+  #   ranef: output of tidy_ranef based on the new formula and old data
+  #   old_ranef: same as 'ranef' but based on the original formula
   draws <- list()
   px <- check_prefix(bterms)
   ranef <- subset2(ranef, ls = px)
@@ -441,29 +449,37 @@ extract_draws_re <- function(bterms, samples, sdata, data, ranef, old_ranef,
   groups <- unique(ranef$group)
   old_ranef <- subset2(old_ranef, ls = px)
   # assigning S4 objects requires initialisation of list elements
-  draws[c("Z", "Zmo", "Zcs", "Zme")] <- list(named_list(groups))
+  draws[c("Z", "Zsp", "Zcs")] <- list(named_list(groups))
   for (g in groups) {
-    new_r <- subset2(ranef, group = g)
-    r_pars <- paste0("^r_", g, usc(usc(p)), "\\[")
-    r <- get_samples(samples, r_pars)
-    if (is.null(r)) {
+    # prepare general variables related to group g
+    sub_ranef <- subset2(ranef, group = g)
+    sub_old_ranef <- subset2(old_ranef, group = g)
+    new_levels <- attr(ranef, "levels")[[g]]
+    old_levels <- attr(old_ranef, "levels")[[g]]
+    new_by_per_level <- attr(new_levels, "by")
+    old_by_per_level <- attr(old_levels, "by")
+    really_new_levels <- setdiff(new_levels, old_levels)
+    nlevels <- length(old_levels) 
+    nranef <- nrow(sub_ranef)
+    # prepare samples of group-level effects
+    rpars <- paste0("^r_", g, usc(usc(p)), "\\[")
+    rsamples <- get_samples(samples, rpars)
+    if (is.null(rsamples)) {
       stop2(
         "Group-level effects for each level of group ", 
         "'", g, "' not found. Please set 'save_ranef' to ", 
         "TRUE when fitting your model."
       )
     }
-    nlevels <- length(attr(old_ranef, "levels")[[g]]) 
-    old_r <- subset2(old_ranef, group = g)
-    used_re <- match(new_r$coef, old_r$coef)
-    used_re_pars <- outer(seq_len(nlevels), (used_re - 1) * nlevels, "+")
-    used_re_pars <- as.vector(used_re_pars)
-    r <- r[, used_re_pars, drop = FALSE]
-    nranef <- nrow(new_r)
-    gtype <- new_r$gtype[1]
-    id <- new_r$id[1]
+    new_rpars <- match(sub_ranef$coef, sub_old_ranef$coef)
+    new_rpars <- outer(seq_len(nlevels), (new_rpars - 1) * nlevels, "+")
+    new_rpars <- as.vector(new_rpars)
+    rsamples <- rsamples[, new_rpars, drop = FALSE]
+    # prepare data required for indexing parameters
+    gtype <- sub_ranef$gtype[1]
+    id <- sub_ranef$id[1]
     if (gtype == "mm") {
-      ngf <- length(new_r$gcall[[1]]$groups)
+      ngf <- length(sub_ranef$gcall[[1]]$groups)
       gf <- sdata[paste0("J_", id, "_", seq_len(ngf))]
       weights <- sdata[paste0("W_", id, "_", seq_len(ngf))]
     } else {
@@ -471,126 +487,140 @@ extract_draws_re <- function(bterms, samples, sdata, data, ranef, old_ranef,
       weights <- list(rep(1, length(gf[[1]])))
     }
     # incorporate new gf levels
-    new_r_levels <- vector("list", length(gf))
+    new_rsamples <- vector("list", length(gf))
     max_level <- nlevels
     for (i in seq_along(gf)) {
       has_new_levels <- any(gf[[i]] > nlevels)
       if (has_new_levels) {
         if (sample_new_levels %in% c("old_levels", "gaussian")) {
-          new_levels <- sort(setdiff(gf[[i]], seq_len(nlevels)))
-          new_r_levels[[i]] <- matrix(
-            nrow = nrow(r), ncol = nranef * length(new_levels)
+          new_indices <- sort(setdiff(gf[[i]], seq_len(nlevels)))
+          new_rsamples[[i]] <- matrix(
+            nrow = nrow(rsamples), ncol = nranef * length(new_indices)
           )
           if (sample_new_levels == "old_levels") {
-            for (j in seq_along(new_levels)) {
+            for (j in seq_along(new_indices)) {
               # choose a person to take the group-level effects from
-              take_level <- sample(seq_len(nlevels), 1)
+              if (length(old_by_per_level)) {
+                new_by <- new_by_per_level[new_levels == really_new_levels[j]]
+                possible_levels <- old_levels[old_by_per_level == new_by]
+                possible_levels <- which(old_levels %in% possible_levels)
+                take_level <- sample(possible_levels, 1)
+              } else {
+                take_level <- sample(seq_len(nlevels), 1)
+              }
               for (k in seq_len(nranef)) {
                 take <- (k - 1) * nlevels + take_level
-                new_r_levels[[i]][, (j - 1) * nranef + k] <- r[, take]
+                new_rsamples[[i]][, (j - 1) * nranef + k] <- rsamples[, take]
               }
             }
           } else if (sample_new_levels == "gaussian") {
-            # extract hyperparameters used to compute the covariance matrix
-            sd_pars <- paste0("sd_", g, usc(usc(p)), "__", new_r$coef)
-            sd_samples <- get_samples(samples, sd_pars, exact = TRUE)
-            cor_type <- paste0("cor_", g)
-            cor_pars <- paste0(usc(p, "suffix"), new_r$coef)
-            cor_pars <- get_cornames(cor_pars, type = cor_type, brackets = FALSE)
-            cor_samples <- matrix(
-              0, nrow = nrow(sd_samples), ncol = length(cor_pars)
-            )
-            for (j in seq_along(cor_pars)) {
-              if (cor_pars[j] %in% colnames(samples)) {
-                cor_samples[, j] <- get_samples(
-                  samples, cor_pars[j], exact = TRUE
-                )
+            for (j in seq_along(new_indices)) {
+              # extract hyperparameters used to compute the covariance matrix
+              if (length(old_by_per_level)) {
+                new_by <- new_by_per_level[new_levels == really_new_levels[j]]
+                rnames <- as.vector(get_rnames(sub_ranef, bylevels = new_by))
+              } else {
+                rnames <- get_rnames(sub_ranef)
               }
-            }
-            # compute the covariance matrix
-            cov_matrix <- get_cov_matrix(sd_samples, cor_samples)
-            for (j in seq_along(new_levels)) {
+              sd_pars <- paste0("sd_", g, "__", rnames)
+              sd_samples <- get_samples(samples, sd_pars, exact = TRUE)
+              cor_type <- paste0("cor_", g)
+              cor_pars <- get_cornames(rnames, cor_type, brackets = FALSE)
+              cor_samples <- matrix(
+                0, nrow = nrow(sd_samples), ncol = length(cor_pars)
+              )
+              for (k in seq_along(cor_pars)) {
+                if (cor_pars[k] %in% colnames(samples)) {
+                  cor_samples[, k] <- get_samples(
+                    samples, cor_pars[k], exact = TRUE
+                  )
+                }
+              }
+              cov_matrix <- get_cov_matrix(sd_samples, cor_samples)
               # sample new levels from the normal distribution
               # implied by the covariance matrix
               indices <- ((j - 1) * nranef + 1):(j * nranef)
-              new_r_levels[[i]][, indices] <- t(apply(
+              new_rsamples[[i]][, indices] <- t(apply(
                 cov_matrix, 1, rmulti_normal, 
                 n = 1, mu = rep(0, length(sd_pars))
               ))
             }
           }
-          max_level <- max_level + length(new_levels)
+          max_level <- max_level + length(new_indices)
         } else if (sample_new_levels == "uncertainty") {
-          new_r_levels[[i]] <- matrix(nrow = nrow(r), ncol = nranef)
+          new_rsamples[[i]] <- matrix(nrow = nrow(rsamples), ncol = nranef)
           for (k in seq_len(nranef)) {
             # sample values for the new level
             indices <- ((k - 1) * nlevels + 1):(k * nlevels)
-            new_r_levels[[i]][, k] <- apply(
-              r[, indices, drop = FALSE], 1, sample, size = 1
+            new_rsamples[[i]][, k] <- apply(
+              rsamples[, indices, drop = FALSE], 1, sample, size = 1
             )
           }
           max_level <- max_level + 1
           gf[[i]][gf[[i]] > nlevels] <- max_level
         }
       } else { 
-        new_r_levels[[i]] <- matrix(nrow = nrow(r), ncol = 0)
+        new_rsamples[[i]] <- matrix(nrow = nrow(rsamples), ncol = 0)
       }
     }
-    new_r_levels <- do.call(cbind, new_r_levels)
+    new_rsamples <- do.call(cbind, new_rsamples)
     # we need row major instead of column major order
     sort_levels <- ulapply(seq_len(nlevels),
-      function(l) seq(l, ncol(r), nlevels)
+      function(l) seq(l, ncol(rsamples), nlevels)
     )
-    r <- cbind(r[, sort_levels, drop = FALSE], new_r_levels)
+    rsamples <- rsamples[, sort_levels, drop = FALSE]
+    # new samples are already in row major order
+    rsamples <- cbind(rsamples, new_rsamples)
     levels <- unique(unlist(gf))
-    r <- subset_levels(r, levels, nranef)
-    # monotonic group-level terms
-    new_r_mo <- subset2(new_r, type = "mo")
-    if (nrow(new_r_mo)) {
+    rsamples <- subset_levels(rsamples, levels, nranef)
+    # special group-level terms (mo, me, mi)
+    sub_ranef_sp <- subset2(sub_ranef, type = "sp")
+    if (nrow(sub_ranef_sp)) {
       Z <- matrix(1, length(gf[[1]])) 
-      draws[["Zmo"]][[g]] <- prepare_Z(Z, gf, max_level, weights)
-      for (co in new_r_mo$coef) {
-        take <- which(new_r$coef == co & new_r$type == "mo")
+      draws[["Zsp"]][[g]] <- prepare_Z(Z, gf, max_level, weights)
+      for (co in sub_ranef_sp$coef) {
+        take <- which(sub_ranef$coef == co & sub_ranef$type == "sp")
         take <- take + nranef * (seq_along(levels) - 1)
-        draws[["rmo"]][[co]][[g]] <- r[, take, drop = FALSE]
-      }
-    }
-    # noise-free group-level terms
-    new_r_me <- subset2(new_r, type = "me")
-    if (nrow(new_r_me)) {
-      Z <- matrix(1, length(gf[[1]]))
-      draws[["Zme"]][[g]] <- prepare_Z(Z, gf, max_level, weights)
-      for (co in new_r_me$coef) {
-        take <- which(new_r$coef == co & new_r$type == "me")
-        take <- take + nranef * (seq_along(levels) - 1)
-        draws[["rme"]][[co]][[g]] <- r[, take, drop = FALSE]
+        draws[["rsp"]][[co]][[g]] <- rsamples[, take, drop = FALSE]
       }
     }
     # category specific group-level terms
-    new_r_cs <- subset2(new_r, type = "cs")
-    if (nrow(new_r_cs)) {
+    sub_ranef_cs <- subset2(sub_ranef, type = "cs")
+    if (nrow(sub_ranef_cs)) {
       # all categories share the same Z matrix
-      cn1 <- new_r_cs$cn[grepl("\\[1\\]$", new_r_cs$coef)]
-      Znames <- paste0("Z_", id, usc(p), "_", cn1)
+      take <- grepl("\\[1\\]$", sub_ranef_cs$coef)
+      Znames <- paste0(
+        "Z_", sub_ranef_cs$id[take], usc(p), "_", sub_ranef_cs$cn[take]
+      )
       Z <- do.call(cbind, sdata[Znames])
       draws[["Zcs"]][[g]] <- prepare_Z(Z, gf, max_level, weights)
       for (i in seq_len(sdata$ncat - 1)) {
         index <- paste0("\\[", i, "\\]$")
-        take <- which(grepl(index, new_r$coef) & new_r$type == "cs")
+        take <- which(grepl(index, sub_ranef$coef) & sub_ranef$type == "cs")
         take <- as.vector(outer(take, nranef * (seq_along(levels) - 1), "+"))
-        draws[["rcs"]][[g]][[i]] <- r[, take, drop = FALSE]
+        draws[["rcs"]][[g]][[i]] <- rsamples[, take, drop = FALSE]
       }
     }
     # basic group-level effects
-    new_r_basic <- subset2(new_r, type = "")
-    if (nrow(new_r_basic)) {
-      Znames <- paste0("Z_", id, usc(p), "_", new_r_basic$cn)
-      Z <- do.call(cbind, sdata[Znames])
+    sub_ranef_basic <- subset2(sub_ranef, type = c("", "mmc"))
+    if (nrow(sub_ranef_basic)) {
+      Znames <- paste0(
+        "Z_", sub_ranef_basic$id, usc(p), "_", sub_ranef_basic$cn
+      )
+      if (sub_ranef_basic$gtype[1] == "mm") {
+        ng <- length(sub_ranef_basic$gcall[[1]]$groups)
+        Z <- vector("list", ng)
+        for (k in seq_len(ng)) {
+          Z[[k]] <- do.call(cbind, sdata[paste0(Znames, "_", k)])
+        }
+      } else {
+        Z <- do.call(cbind, sdata[Znames])
+      }
       draws[["Z"]][[g]] <- prepare_Z(Z, gf, max_level, weights)
-      take <- which(!nzchar(new_r$type))
+      take <- which(sub_ranef$type %in% c("", "mmc"))
       take <- as.vector(outer(take, nranef * (seq_along(levels) - 1), "+"))
-      r <- r[, take, drop = FALSE]
-      draws[["r"]][[g]] <- r
+      rsamples <- rsamples[, take, drop = FALSE]
+      draws[["r"]][[g]] <- rsamples
     }
   }
   draws
@@ -657,15 +687,21 @@ extract_draws_autocor <- function(bterms, samples, sdata, new = FALSE, ...) {
   draws
 }
 
-extract_draws_data <- function(bterms, sdata, ...) {
+extract_draws_data <- function(bterms, sdata, stanvars = NULL, ...) {
   # extract data mainly related to the response variable
+  # Args
+  #   stanvars: *names* of variables stored in slot 'stanvars'
   vars <- c(
     "Y", "trials", "ncat", "se", "weights", 
     "dec", "cens", "rcens", "lb", "ub"
   )
   resp <- usc(combine_prefix(bterms))
-  draws <- sdata[paste0(vars, resp)]
-  rmNULL(draws, recursive = FALSE)
+  draws <- rmNULL(sdata[paste0(vars, resp)], recursive = FALSE)
+  if (length(stanvars)) {
+    stopifnot(is.character(stanvars))
+    draws[stanvars] <- sdata[stanvars]
+  }
+  draws
 }
 
 pseudo_draws_for_mixture <- function(draws, comp, sample_ids = NULL) {
@@ -686,7 +722,7 @@ pseudo_draws_for_mixture <- function(draws, comp, sample_ids = NULL) {
   out$f$fun <- out$f$family
   for (dp in valid_dpars(out$f)) {
     out$dpars[[dp]] <- draws$dpars[[paste0(dp, comp)]]
-    if (!is.null(sample_ids)) {
+    if (length(sample_ids) && length(out$dpars[[dp]]) > 1L) {
       out$dpars[[dp]] <- p(out$dpars[[dp]], sample_ids, row = TRUE)
     }
   }
@@ -706,19 +742,33 @@ subset_levels <- function(x, levels, nranef) {
   x[, take_levels, drop = FALSE]
 }
 
-prepare_Z <- function(Z, gf, max_level = max(unlist(gf)),
-                      weights = list(rep(1, length(gf[[1]])))) {
+prepare_Z <- function(Z, gf, max_level = NULL, weights = NULL) {
   # prepare group-level design matrices for use in linear_predictor
   # Args:
-  #   Z: matrix to be prepared
-  #   gf: list of vectors containing grouping factor values
+  #   Z: (list of) matrices to be prepared
+  #   gf: (list of) vectors containing grouping factor values
+  #   weights: optional (list of) weights of the same length as gf
   #   max_level: maximal level of gf
-  #   weights: optional list of weights of the same length as gf
-  nranef <- ncol(Z)
+  if (!is.list(Z)) {
+    Z <- list(Z)
+  }
+  if (!is.list(gf)) {
+    gf <- list(gf)
+  }
+  if (is.null(weights)) {
+    weights <- rep(1, length(gf[[1]]))
+  }
+  if (!is.list(weights)) {
+    weights <- list(weights)
+  }
+  if (is.null(max_level)) {
+    max_level <- max(unlist(gf))
+  }
   levels <- unique(unlist(gf))
+  nranef <- ncol(Z[[1]])
   Z <- mapply(
-    expand_matrix, x = gf, weights = weights,
-    MoreArgs = nlist(A = Z, max_level)
+    expand_matrix, A = Z, x = gf, weights = weights,
+    MoreArgs = nlist(max_level)
   )
   Z <- Reduce("+", Z)
   subset_levels(Z, levels, nranef)
@@ -729,15 +779,15 @@ expand_matrix <- function(A, x, max_level = max(x), weights = 1) {
   # Args:
   #   A: matrix to be expanded
   #   x: levels to expand the matrix
+  #   max_level: maximal number of levels that x can take on
   #   weights: weights to apply to rows of A before expanding
-  #   nlevels: number of levels that x can take on
   # Returns:
   #   A sparse matrix of dimension nrow(A) x (ncol(A) * max_level)
   stopifnot(is.matrix(A))
   stopifnot(length(x) == nrow(A))
   stopifnot(all(is_wholenumber(x) & x > 0))
   stopifnot(length(weights) %in% c(1, nrow(A), prod(dim(A))))
-  A <- A * weights
+  A <- A * as.vector(weights)
   K <- ncol(A)
   i <- rep(seq_along(x), each = K)
   make_j <- function(n, K, x) K * (x[n] - 1) + 1:K
@@ -761,3 +811,12 @@ is.mvbrmsdraws <- function(x) {
   inherits(x, "mvbrmsdraws")
 }
 
+is.bdrawsl <- function(x) {
+  inherits(x, "bdrawsl")
+}
+
+is.bdrawsnl <- function(x) {
+  inherits(x, "bdrawsnl")
+}
+
+
