@@ -24,10 +24,62 @@ restructure <- function(x, rstr_summary = FALSE) {
     # also added the rstan version in brms 1.5.0
     x$version <- list(brms = x$version)
   }
-  version <- x$version$brms
-  if (version >= "2.0.0" || isTRUE(attr(x, "restructured"))) {
-    return(x) 
+  if (!isTRUE(attr(x, "restructured"))) {
+    if (x$version$brms < "2.0.0") {
+      x <- restructure_v1(x)
+    }
+    x <- restructure_v2(x)
   }
+  stan_env <- attributes(x$fit)$.MISC
+  if (rstr_summary && exists("summary", stan_env)) {
+    stan_summary <- get("summary", stan_env)
+    old_parnames <- rownames(stan_summary$msd)
+    if (!identical(old_parnames, parnames(x))) {
+      # do not rename parameters in the cached summary
+      # just let rstan compute the summary again
+      remove("summary", pos = stan_env)
+    }
+  }
+  structure(x, restructured = TRUE)
+}
+
+restructure_v2 <- function(x) {
+  # restructure models fitted with brms 2.x
+  version <- x$version$brms
+  pars <- parnames(x)
+  bterms <- parse_bf(formula(x))
+  if (version <= "2.1.1") {
+    x <- do_renaming(x, change_old_bsp(pars))
+  }
+  if (version <= "2.1.2") {
+    if ("weibull" %in% family_names(x)) {
+      stop_parameterization_changed("weibull", "2.1.3")
+    }
+  }
+  if (version <= "2.1.7") {
+    if ("exgaussian" %in% family_names(x)) {
+      stop_parameterization_changed("exgaussian", "2.1.8")
+    }
+  }
+  if (version <= "2.1.8") {
+    # added 'by' variables to grouping terms
+    x$ranef <- tidy_ranef(bterms, model.frame(x))
+    # reworked 'me' terms (#372)
+    meef <- tidy_meef(bterms, model.frame(x))
+    if (isTRUE(nrow(meef) > 0)) {
+      warning2(
+        "Measurement error ('me') terms have been reworked ",
+        "in version 2.1.9. I strongly recommend refitting your ",
+        "model with the latest version of brms."
+      )
+    }
+  }
+  x
+}
+
+restructure_v1 <- function(x) {
+  # restructure models fitted with brms 1.x
+  version <- x$version$brms
   if (version < "1.0.0") {
     warning2(
       "Models fitted with brms < 1.0 are no longer offically ",
@@ -55,28 +107,21 @@ restructure <- function(x, rstr_summary = FALSE) {
   if (version <= "0.10.0.9000") {
     if (length(bterms$dpars$mu$nlpars)) {
       # nlpar and group have changed positions
-      change <- change_old_re(x$ranef, pars = parnames(x),
-                              dims = x$fit@sim$dims_oi)
+      change <- change_old_re(x$ranef, parnames(x), x$fit@sim$dims_oi)
       x <- do_renaming(x, change)
     }
   }
   if (version < "1.0.0") {
     # double underscores were added to group-level parameters
-    change <- change_old_re2(x$ranef, pars = parnames(x),
-                             dims = x$fit@sim$dims_oi)
+    change <- change_old_re2(x$ranef, parnames(x), x$fit@sim$dims_oi)
     x <- do_renaming(x, change)
   }
   if (version <= "1.0.1") {
     # names of spline parameters had to be changed after
     # allowing for multiple covariates in one spline term
-    change <- change_old_sm(bterms, pars = parnames(x),
-                            dims = x$fit@sim$dims_oi)
+    change <- change_old_sm(bterms, parnames(x), x$fit@sim$dims_oi)
     x <- do_renaming(x, change)
   }
-  if (version <= "1.2.0") {
-    x$ranef$type[x$ranef$type == "mono"] <- "mo"
-    x$ranef$type[x$ranef$type == "cse"] <- "cs"
-  }
   if (version <= "1.8.0") {
     att <- attributes(x$exclude)
     if (is.null(att$save_ranef)) {
@@ -101,17 +146,7 @@ restructure <- function(x, rstr_summary = FALSE) {
     change <- change_old_categorical(bterms, x$data, pars = parnames(x))
     x <- do_renaming(x, change)
   }
-  stan_env <- attributes(x$fit)$.MISC
-  if (rstr_summary && exists("summary", stan_env)) {
-    stan_summary <- get("summary", stan_env)
-    old_parnames <- rownames(stan_summary$msd)
-    if (!identical(old_parnames, parnames(x))) {
-      # do not rename parameters in the cached summary
-      # just let rstan compute the summary again
-      remove("summary", pos = stan_env)
-    }
-  }
-  structure(x, restructured = TRUE)
+  x
 }
 
 restructure_formula <- function(formula, nonlinear = NULL) {
@@ -148,7 +183,7 @@ change_old_re <- function(ranef, pars, dims) {
   #   dims: dimension of parameters
   # Returns:
   #   a list whose elements can be interpreted by do_renaming
-  change <- list()
+  out <- list()
   for (id in unique(ranef$id)) {
     r <- subset2(ranef, id = id)
     g <- r$group[1]
@@ -158,8 +193,8 @@ change_old_re <- function(ranef, pars, dims) {
     old_sd_names <- paste0("sd_", nlpar, "_", g, "_", r$coef)
     new_sd_names <- paste0("sd_", g, "_", nlpar, "_", r$coef)
     for (i in seq_along(old_sd_names)) {
-      change <- lc(change, 
-        change_simple(old_sd_names[i], new_sd_names[i], pars, dims)
+      lc(out) <- change_simple(
+        old_sd_names[i], new_sd_names[i], pars, dims
       )
     }
     # rename cor-parameters
@@ -172,8 +207,8 @@ change_old_re <- function(ranef, pars, dims) {
       type = paste0("cor_", nlpar, "_", g)
     )
     for (i in seq_along(old_cor_names)) {
-      change <- lc(change, 
-        change_simple(old_cor_names[i], new_cor_names[i], pars, dims)
+      lc(out) <- change_simple(
+        old_cor_names[i], new_cor_names[i], pars, dims
       )
     } 
     # rename r-parameters
@@ -182,14 +217,12 @@ change_old_re <- function(ranef, pars, dims) {
     levels <- gsub("[ \t\r\n]", ".", attr(ranef, "levels")[[g]])
     index_names <- make_index_names(levels, r$coef, dim = 2)
     new_r_names <- paste0(new_r_name, index_names)
-    change <- lc(change, 
-      change_simple(
-        old_r_name, new_r_names, pars, dims, 
-        pnames = new_r_name
-      )
+    lc(out) <- change_simple(
+      old_r_name, new_r_names, pars, dims, 
+      pnames = new_r_name
     )
   }
-  change
+  out
 }
 
 change_old_re2 <- function(ranef, pars, dims) {
@@ -202,7 +235,7 @@ change_old_re2 <- function(ranef, pars, dims) {
   #   dims: dimension of parameters
   # Returns:
   #   a list whose elements can be interpreted by do_renaming
-  change <- list()
+  out <- list()
   for (id in unique(ranef$id)) {
     r <- subset2(ranef, id = id)
     g <- r$group[1]
@@ -211,9 +244,7 @@ change_old_re2 <- function(ranef, pars, dims) {
     old_sd_names <- paste0("sd_", g, "_", nlpars_usc, r$coef)
     new_sd_names <- paste0("sd_", g, "__", nlpars_usc, r$coef)
     for (i in seq_along(old_sd_names)) {
-      change <- lc(change, 
-        change_simple(old_sd_names[i], new_sd_names[i], pars, dims)
-      )
+      lc(out) <- change_simple(old_sd_names[i], new_sd_names[i], pars, dims)
     }
     # rename cor-parameters
     new_cor_names <- get_cornames(
@@ -225,8 +256,7 @@ change_old_re2 <- function(ranef, pars, dims) {
       brackets = FALSE, sep = "_"
     )
     for (i in seq_along(old_cor_names)) {
-      change <- lc(change, 
-        change_simple(old_cor_names[i], new_cor_names[i], pars, dims))
+      lc(out) <- change_simple(old_cor_names[i], new_cor_names[i], pars, dims)
     } 
     # rename r-parameters
     for (nlpar in unique(r$nlpar)) {
@@ -236,22 +266,20 @@ change_old_re2 <- function(ranef, pars, dims) {
       levels <- gsub("[ \t\r\n]", ".", attr(ranef, "levels")[[g]])
       index_names <- make_index_names(levels, sub_r$coef, dim = 2)
       new_r_names <- paste0(new_r_name, index_names)
-      change <- lc(change, 
-        change_simple(
-          old_r_name, new_r_names, pars, dims, 
-          pnames = new_r_name
-        )
+      lc(out) <- change_simple(
+        old_r_name, new_r_names, pars, dims, 
+        pnames = new_r_name
       )
     }
   }
-  change
+  out
 }
 
 change_old_sm <- function(bterms, pars, dims) {
   # change names of spline parameters fitted with brms <= 1.0.1
   # this became necessary after allowing smooths with multiple covariates
   .change_old_sm <- function(bt) {
-    change <- list()
+    out <- list()
     sm_labels <- get_sm_labels(bt)
     if (length(sm_labels)) {
       p <- usc(combine_prefix(bt), "suffix")
@@ -262,100 +290,98 @@ change_old_sm <- function(bterms, pars, dims) {
       old_s_pars <- paste0("s_", old_smooths)
       new_s_pars <- paste0("s_", new_smooths, "_1")
       for (i in seq_along(old_smooths)) {
-        change <- lc(change,
-          change_simple(old_sds_pars[i], new_sds_pars[i], pars, dims)
-        )
+        lc(out) <- change_simple(old_sds_pars[i], new_sds_pars[i], pars, dims)
         dim_s <- dims[[old_s_pars[i]]]
         if (!is.null(dim_s)) {
           new_s_par_indices <- paste0(new_s_pars[i], "[", seq_len(dim_s), "]")
-          change <- lc(change,
-            change_simple(
-              old_s_pars[i], new_s_par_indices, pars, dims,
-              pnames = new_s_pars[i]
-            )
+          lc(out) <- change_simple(
+            old_s_pars[i], new_s_par_indices, pars, dims,
+            pnames = new_s_pars[i]
           )
         }
       }
     }
-    return(change)
+    return(out)
   }
   
-  change <- list()
+  out <- list()
   if (is.mvbrmsterms(bterms)) {
     for (r in bterms$responses) {
-      change <- c(change, .change_old_sm(bterms$terms[[r]]$dpars$mu))
+      c(out) <- .change_old_sm(bterms$terms[[r]]$dpars$mu)
     }
   } else if (is.brmsterms(bterms)) {
     for (dp in names(bterms$dpars)) {
       bt <- bterms$dpars[[dp]]
       if (length(bt$nlpars)) {
         for (nlp in names(bt$nlpars)) {
-          change <- c(change, .change_old_sm(bt$nlpars[[nlp]]))
+          c(out) <- .change_old_sm(bt$nlpars[[nlp]])
         }
       } else {
-        change <- c(change, .change_old_sm(bt))
+        c(out) <- .change_old_sm(bt)
       }
     }
   }
-  change
+  out
 }
 
 change_old_mo <- function(bterms, data, pars) {
   # change names of monotonic effects fitted with brms <= 1.9.0
   # this became necessary after implementing monotonic interactions
   .change_old_mo <- function(bt) {
-    change <- list()
-    monef <- get_mo_labels(bt, data)
-    if (!length(monef)) {
-      return(change)
+    out <- list()
+    spef <- tidy_spef(bt, data)
+    has_mo <- lengths(spef$call_mo) > 0
+    if (!any(has_mo)) {
+      return(out)
     }
+    spef <- spef[has_mo, ]
     p <- usc(combine_prefix(bt))
     bmo_prefix <- paste0("bmo", p, "_")
     bmo_regex <- paste0("^", bmo_prefix, "[^_]+$")
     bmo_old <- pars[grepl(bmo_regex, pars)]
-    bmo_new <- paste0(bmo_prefix, rename(monef))
+    bmo_new <- paste0(bmo_prefix, spef$coef)
     if (length(bmo_old) != length(bmo_new)) {
       stop2("Restructuring failed. Please refit your ", 
             "model with the latest version of brms.")
     }
     for (i in seq_along(bmo_old)) {
       pos <- grepl(paste0("^", bmo_old[i]), pars)
-      change <- lc(change, nlist(pos, fnames = bmo_new[i]))
+      lc(out) <- clist(pos, fnames = bmo_new[i])
     }
     simo_regex <- paste0("^simplex", p, "_[^_]+$")
     simo_old_all <- pars[grepl(simo_regex, pars)]
     simo_index <- get_matches("\\[[[:digit:]]+\\]$", simo_old_all)
     simo_old <- unique(sub("\\[[[:digit:]]+\\]$", "", simo_old_all))
-    simo_coef <- get_simo_labels(monef)
+    simo_coef <- get_simo_labels(spef)
     for (i in seq_along(simo_old)) {
       regex_pos <- paste0("^", simo_old[i])
       pos <- grepl(regex_pos, pars)
       simo_new <- paste0("simo", p, "_", simo_coef[i])
       simo_index_part <- simo_index[grepl(regex_pos, simo_old_all)]
       simo_new <- paste0(simo_new, simo_index_part)
-      change <- lc(change, nlist(pos, fnames = simo_new))
+      lc(out) <- clist(pos, fnames = simo_new)
     }
-    return(change)
+    return(out)
   }
   
-  change <- list()
+  out <- list()
   if (is.mvbrmsterms(bterms)) {
     for (r in bterms$responses) {
-      change <- c(change, .change_old_mo(bterms$terms[[r]]$dpars$mu))
+      c(out) <- .change_old_mo(bterms$terms[[r]]$dpars$mu)
     }
   } else if (is.brmsterms(bterms)) {
     for (dp in names(bterms$dpars)) {
       bt <- bterms$dpars[[dp]]
       if (length(bt$nlpars)) {
         for (nlp in names(bt$nlpars)) {
-          change <- c(change, .change_old_mo(bt$nlpars[[nlp]]))
+          c(out) <- .change_old_mo(bt$nlpars[[nlp]])
         }
       } else {
-        change <- c(change, .change_old_mo(bt))
+        c(out) <- .change_old_mo(bt)
       }
     }
   }
-  change
+  out
 }
 
 change_old_categorical <- function(bterms, data, pars) {
@@ -384,15 +410,31 @@ change_old_categorical <- function(bterms, data, pars) {
   list(nlist(pos, fnames = new_pars[pos]))
 }
 
-change_simple <- function(oldname, fnames, pars, dims,
-                          pnames = fnames) {
+change_old_bsp <- function(pars) {
+  # as of brms 2.2 'mo' and 'me' terms are handled together
+  pos <- grepl("^(bmo|bme)_", pars)
+  if (!any(pos)) return(list()) 
+  fnames <- gsub("^(bmo|bme)_", "bsp_", pars[pos])
+  list(nlist(pos, fnames))
+}
+
+change_simple <- function(oldname, fnames, pars, dims, pnames = fnames) {
   # helper function for very simple renaming
   # only used in renaming of old models
   pos <- grepl(paste0("^", oldname), pars)
   if (any(pos)) {
     out <- nlist(pos, oldname, pnames, fnames, dims = dims[[oldname]])
+    class(out) <- c("clist", "list")
   } else {
     out <- NULL
   }
   out
 }
+
+stop_parameterization_changed <- function(family, version) {
+  stop2(
+    "The parameterization of '", family, "' models has changed in brms ",
+    version, " to be consistent with other model classes. ", 
+    "Please refit your model with the current version of brms."
+  )
+}
